/*

  seng.c: A Search ENGine.
  Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 

#include "seng.h"


/****************************************************************************/

void ReadFile(char *filename, HashTable H){


 FILE *fp;
 char line[LINE_LENGTH];
 const char delimiters[] = " .,;:!?-(){}[]'\"\n\t"; /*For strsep: These divide words.*/
 char *running;                        /*For strsep: The moving pointer it needs.*/
 char *token;                          /*For strsep: Another word, another token.*/
 ElementType CurrentElement;

 if( (fp = fopen(filename, "r")) == NULL){
   fprintf(stderr, "Error while opening file %s.\n", filename);
   exit(1);
 }

 else{

   while( !feof(fp) ){
     
     /*Let's fill the string line[LINE_LENGTH] with contents of the file.*/
     
     if( fgets(line, LINE_LENGTH, fp) ){
       /* If end-of-file occurs before any characters are read, fgets re-
	  turns NULL and the buffer contents is unchanged. With this if clause
	  we handle this situation.*/

       /*We are going to use strsep -the reentrant BSD successor of strtok-
	 to separate the words of every line.*/
       /*We could use a temporary string with a copy of the original line here,
	 since strsep will change the line, but -since we are not going to need
	 the line anymore- we avoid it to gain Time.*/
       /*So we just use the moving pointer strsep needs.*/
       
       running = line;

       while( token = strsep(&running, delimiters) ){
	 
	 /*So that we don't store empty tokens.*/
	 if( strcmp("", token)!=0 ){
	   strcpy(CurrentElement.word, token);
	   strcpy(CurrentElement.filename, filename);
	   Insert(CurrentElement, H);
	 }
       }
     } 
   }  

   if (fclose(fp) != 0){
     fprintf(stderr, "Error while closing file %s.\n", filename);
     exit(1);
   }
 }

}

/****************************************************************************/

/****************************************************************************/

void SearchFor(char *word2find, char *secondword, char *status, HashTable H){

  int foundatleastonce=0; /*A flag.*/
  Position P;
  List L;

  Position secondP;
  List secondL;
  
  /*ONE*/
  /*___________________________________________________________________*/

  if(strcmp(status, "ONE")==0){
    printf("Searching for: %s\n", word2find);
    
    L = H->ArrayOfLists[ Hash( word2find, H->TableSize ) ];
    P = L->Next;
    
    while( P != NULL ){
      if( strcmp(P->Element.word, word2find)==0 ){
	foundatleastonce=1;
	printf("Found in file: %s\n", P->Element.filename);
      }
      P = P->Next;
    }

    if(!foundatleastonce)
      printf("Not found in any of the files.\n");
  }

  
  /*AND*/
  /*___________________________________________________________________*/

  
  if(strcmp(status, "AND")==0){
    printf("Searching for: %s AND %s\n", word2find, secondword);
    
    L = H->ArrayOfLists[ Hash( word2find, H->TableSize ) ];
    P = L->Next;
    
    while( P != NULL ){

      if( strcmp(P->Element.word, word2find)==0 ){
	    
	secondL = H->ArrayOfLists[ Hash( secondword, H->TableSize ) ];
	secondP = secondL->Next;
	
	while( secondP !=NULL ){

	  if( strcmp(secondP->Element.word, secondword)==0 ){

	    if( strcmp(P->Element.filename, secondP->Element.filename)==0 ){
	      foundatleastonce=1;
	      printf("Found in file: %s\n", P->Element.filename);
	    }
	    
	  }
	  secondP = secondP->Next;
	}
      }
      P = P->Next;
    }
    
    if(!foundatleastonce)
      printf("Not found in any of the files.\n");
      
  }
  
}


/****************************************************************************/
