/*

  seng.c: A Search ENGine.
  Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
#include "seng.h"

/****************************************************************************/
HashTable InitializeTable(int TableSize){

  HashTable H;
  int i;

  /* Allocate table */
  H = malloc( sizeof( struct HashTbl ) );
  if( H == NULL ){
    fprintf(stderr, "Error: The memory is full.\n");
    exit(1);
  }

  H->TableSize = TableSize;

  /* Allocate array of lists */
  H->ArrayOfLists = malloc( sizeof( List ) * H->TableSize );
  if( H->ArrayOfLists == NULL ){
    fprintf(stderr, "Error: The memory is full.\n");
    exit(1);
  }
   
  /* Allocate list headers */
  for( i = 0; i < H->TableSize; i++ )
    {
      H->ArrayOfLists[ i ] = malloc( sizeof( struct ListNode ) );
      if( H->ArrayOfLists[ i ] == NULL ){
	fprintf(stderr, "Error: The memory is full.\n");
	exit(1);
      }
      else
	H->ArrayOfLists[ i ]->Next = NULL;
    }
    
  return H;
  
}
 


/****************************************************************************/

/****************************************************************************/
Position Find( ElementType Key, HashTable H ) {
  
  Position P;
  List L;
  
  L = H->ArrayOfLists[ Hash( Key.word, H->TableSize ) ];
  P = L->Next;

  while( P != NULL ){
    /*We want to go to the end of the list, except if we already have this key.*/  
    if( strcmp(P->Element.word, Key.word)==0
	&& strcmp(P->Element.filename, Key.filename)==0 )
      goto alreadyhavekey; /*There was no other way, not even a flag did it.*/

    P = P->Next;
  }
  
 alreadyhavekey:
  ;
  
  return P;
}

/****************************************************************************/

/****************************************************************************/
void Insert( ElementType Key, HashTable H )
{
  Position Pos, NewCell;
  List L;
  
  Pos = Find( Key, H );
  if( Pos == NULL ){
   
    /* Key was not found, we are going to add it. */

    numberofwords = numberofwords + 1;

    NewCell = malloc( sizeof( struct ListNode ) );
    
    if( NewCell == NULL ){
      fprintf(stderr, "Error: The memory is full.\n");
      exit(1);
    }
    else
	{
	  L = H->ArrayOfLists[ Hash( Key.word, H->TableSize ) ];
	  NewCell->Next = L->Next;
	  strcpy(NewCell->Element.word, Key.word);
	  strcpy(NewCell->Element.filename, Key.filename);
	  L->Next = NewCell;
	}
  }
}

/****************************************************************************/

/****************************************************************************/

void PrintTable( HashTable H ){

 Position P;
 List L;
 int i;

 printf("Printing Hash Table status:\n");

 for( i = 0; i < (H->TableSize); i++){
   L = H->ArrayOfLists[ i ];
   P = L->Next;
   
   while( P != NULL ){
     printf("Position: %d\t\t", i);
     printf("Word: %s\t\t", P->Element.word);
     printf("Filename: %s\n", P->Element.filename);
     P = P->Next;
   }
 }
}


/****************************************************************************/


/****************************************************************************/

void DestroyTable( HashTable H ){
  int i;
  
  for( i = 0; i < H->TableSize; i++ ){

    Position P = H->ArrayOfLists[ i ];
    Position Tmp;
    
    while( P != NULL ){

      Tmp = P->Next;
      free( P );
      P = Tmp;
    }
  }
  
  free( H->ArrayOfLists );
  free( H );
}


/****************************************************************************/

/****************************************************************************/

Index Hash(const char *Key, int TableSize){

  /*We are using a modified "modulo prime" hash function since it
    doesn't need much time in calculations and doesn't produce
    way too many collisions (is uniformal enough).
  */

  unsigned int HashVal = 0;
 
  while( *Key != '\0' )
    HashVal = ( HashVal << 5 ) + *Key++;
  
  return HashVal % TableSize;


} 

/****************************************************************************/
