/*

  seng.c: A Search ENGine.
  Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/
 
/*

  seng takes a list of ASCII text files as an input, stores them in the
  memory, prompts for a word or a logical combination of words to search
  for and returns the name(s) of the file(s) containing this or those words
  (commas etc. are ignored and the search is case sensitive).

*/


#include "seng.h"


char word2find[WORD_LENGTH] = "";
char secondword[WORD_LENGTH] = "";
int numberofwords = 0;


/****************************************************************************/


main(int argc, char *argv[]){

  HashTable HT;
  clock_t t1, t2;

  printf("\n");
  printf("seng: A search engine.\n");
  printf("Copyright (C) 2000 Thomas Repantis <darkzero@otenet.gr>\n");
  printf("\n");  

  if (argc == 1){
    printf("Usage: seng [ASCII text files]\n");
    printf("\n");
    exit(1);
  }
  
  else{

    t1 = clock();

    HT = InitializeTable(TABLE_SIZE);
    while(--argc > 0)
      ReadFile(*++argv, HT);
  
    t2 = clock();

    printf("Hash Table initialized and words scanned in %d seconds.\n", 
	   ((t2-t1)/CLOCKS_PER_SEC));
    
    printf("%d words stored.\n", numberofwords);
    printf("\n");

  }
  
  printf("Enter a word to search for, or
 .AND to search for word1 AND word2, or
 .p to print Hash Table status, or
 .h for help, or
 .q to quit.\n");
  printf("\n");
  
  while(strcmp(word2find, ".q") != 0){ 
    
    printf("> ");
    scanf("%s", word2find);

    if(strcmp(word2find, ".h") == 0){
     
      printf("\n");
      printf("Usage: seng [ASCII text files]\n");

      printf("seng takes a list of ASCII text files as an input, stores them in the memory, prompts for a word or a logical combination of words to search for and returns the name(s) of the file(s) containing this or those words (commas etc. are ignored and the search is case sensitive).\n");

      printf("You can enter a word to search for, or
 .AND to search for word1 AND word2, or
 .p to print Hash Table status, or
 .h for help, or
 .q to quit.\n");
      
      printf("This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. seng is licensed under the GNU General Public License. See the file COPYING for details.\n");
      printf("\n");



    }

    else if(strcmp(word2find, ".p") == 0){
      PrintTable(HT);
    }
    
    /*We only provide AND and OR for 2 words, since we do not have infinite
      number of variables to store them, though SearchFor could have variable
      argument lists.*/

    else if(strcmp(word2find, ".AND") == 0){
      
      printf("word1: > "); 
      scanf("%s", word2find);
      printf("AND\n");
      printf("word2: > ");
      scanf("%s", secondword);

      SearchFor(word2find, secondword, "AND", HT);
   
    }

    else if(strcmp(word2find, ".q") == 0){
      DestroyTable(HT);
    }

    else{
      
      SearchFor(word2find, "", "ONE", HT);
	
	}
     
  }
}
/****************************************************************************/

